@@ -204,7 +204,9 @@ If KBYTES is kilo bytes (This value must be float)."
 				      (< (cdr x)
 					 (cdr y))))))))
       (setq i (+ i 1))
-      (message "Collecting cache info...%d%%" (/ (* i 100) num))
+      (elmo-display-progress
+       'elmo-cache-get-sorted-cache-file-list "Collecting cache info..."
+       (/ (* i 100) num))
       (setq dirs (cdr dirs)))
     ret-val))
 
@@ -295,7 +297,9 @@ If KBYTES is kilo bytes (This value must be float)."
 	  (setq ret-val (append ret-val (list (caar nalist)))))
       (setq i (1+ i))
       (setq percent (/ (* i 100) num))
-      (message "Searching...%d%%" percent)
+      (elmo-display-progress
+       'elmo-cache-search-all "Searching..."
+       percent)
       (setq nalist (cdr nalist)))
     ret-val))
 
@@ -395,6 +399,9 @@ Returning its cache buffer."
 	(setq buf (get-buffer-create (format "%s%d" elmo-buffer-cache-name len)))
       (setq buf (elmo-buffer-cache-buffer-get (nth (1- len) elmo-buffer-cache)))
       (setcdr (nthcdr (- len 2) elmo-buffer-cache) nil))
+    (save-excursion
+      (set-buffer buf)
+      (elmo-set-buffer-multibyte nil))
     (setq elmo-buffer-cache
 	  (cons (elmo-buffer-cache-entry-make fld-msg-id buf)
 		elmo-buffer-cache))
@@ -528,7 +535,9 @@ Returning its cache buffer."
 		     gmark))))
 	(setq i (1+ i))
 	(setq percent (/ (* i 100) len))
-	(message "Creating msgdb...%d%%" percent)
+	(elmo-display-progress
+	 'elmo-cache-msgdb-create-as-numlist "Creating msgdb..."
+	 percent)
 	(setq numlist (cdr numlist)))
       (message "Creating msgdb...done.")
       (list overview number-alist mark-alist))))
@@ -677,7 +686,9 @@ Returning its cache buffer."
 					    condition)
 	  (setq ret-val (cons (car msgs) ret-val)))
       (setq i (1+ i))
-      (message "Searching...%d%%" (/ (* i 100) num))
+      (elmo-display-progress
+       'elmo-cache-search "Searching..."
+       (/ (* i 100) num))
       (setq msgs (cdr msgs)))
     (nreverse ret-val)))
 
