@@ -1025,8 +1025,9 @@ If optional argument UNMARK is non-nil, unmark."
 	  (setq count (+ count (car (car set-list))))
 	  (setq ov-str (elmo-imap4-read-contents (process-buffer process)
 						 process))
-	  (message "Getting overview...%d%%" 
-		   (/ (* count 100) length))
+	  (elmo-display-progress
+	   'elmo-imap4-msgdb-create "Getting overview..." 
+	   (/ (* count 100) length))
 	  (setq set-list (cdr set-list)))
 	;; process last one.
 	(with-current-buffer (process-buffer process)
