@@ -143,7 +143,9 @@
 			 new-mark))))))
 	(setq i (1+ i))
 	(setq percent (/ (* i 100) num))
-	(message "Creating msgdb...%d%%" percent)
+	(elmo-display-progress
+	 'elmo-internal-msgdb-create "Creating msgdb..."
+	 percent)
 	(setq numlist (cdr numlist)))
       (message "Creating msgdb...done.")
       (list overview number-alist mark-alist loc-alist))))
@@ -236,7 +238,9 @@
 	  (setq ret-val (append ret-val (list (car (car loc-alist))))))
       (setq i (1+ i))
       (setq percent (/ (* i 100) num))
-      (message "Searching...%d%%" percent)
+      (elmo-display-progress
+       'elmo-internal-search "Searching..."
+       percent)
       (setq loc-alist (cdr loc-alist)))
     ret-val))
 
