@@ -154,7 +154,9 @@
 		     gmark))))
 	(setq i (1+ i))
 	(setq percent (/ (* i 100) len))
-	(message "Creating msgdb...%d%%" percent)
+	(elmo-display-progress
+	 'elmo-localdir-msgdb-create-as-numlist "Creating msgdb..."
+	 percent)
 	(setq numlist (cdr numlist)))
       (message "Creating msgdb...done.")
       (list overview number-alist mark-alist))))
@@ -359,7 +361,9 @@
 					       condition)
 	  (setq ret-val (cons (car msgs) ret-val)))
       (setq i (1+ i))
-      (message "Searching...%d%%" (/ (* i 100) num))
+      (elmo-display-progress
+       'elmo-localdir-search "Searching..."
+       (/ (* i 100) num))
       (setq msgs (cdr msgs)))
     (nreverse ret-val)))
 
@@ -400,7 +404,9 @@
 	    (mapcar 'car onum-alist)))
     (setq total (length flist))
     (while flist
-      (message "Packing...%d%%" (/ (* i 100) total))
+      (elmo-display-progress
+       'elmo-localdir-pack-number "Packing..."
+       (/ (* i 100) total))
       (setq onum (car flist))
       (when (not (eq onum i)) ;; why \=() is wrong..
         (elmo-bind-directory
