@@ -4,7 +4,7 @@
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
 ;; Keywords: mail, net news
-;; Time-stamp: <00/03/22 00:14:31 teranisi>
+;; Time-stamp: <00/04/24 10:19:24 teranisi>
 
 ;; This file is part of ELMO (Elisp Library for Message Orchestration).
 
@@ -225,10 +225,13 @@ This variable should not be used in elsewhere.")
 		       new-mark)))))
 	(setq i (1+ i))
 	(setq percent (/ (* i 100) num))
-	(message "Creating msgdb...%d%%" percent)
+	(elmo-display-progress
+	 'elmo-maildir-msgdb-create "Creating msgdb..."
+	 percent)
 	(setq numlist (cdr numlist)))
       (message "Creating msgdb...done.")
-      (list overview number-alist mark-alist loc-alist))))
+      (elmo-msgdb-sort-by-date
+       (list overview number-alist mark-alist loc-alist)))))
 
 (defalias 'elmo-maildir-msgdb-create-as-numlist 'elmo-maildir-msgdb-create)
 
@@ -428,7 +431,9 @@ file name for maildir directories."
 	    (setq ret-val (append ret-val (list msg-num))))
 	(setq i (1+ i))
 	(setq percent (/ (* i 100) num))
-	(message "Searching...%d%%" percent)
+	(elmo-display-progress
+	 'elmo-maildir-search "Searching..."
+	 percent)
 	(setq msgs (cdr msgs)))
       ret-val)))
 
