@@ -1,23 +1,36 @@
-;;;
 ;;; elmo-msgdb.el -- Message Database for Elmo.
-;;;   Copyright 1998 Yuuichi Teranishi <teranisi@gohome.org>
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;
 
+;; Copyright 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+
+;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;; Keywords: mail, net news
+;; Time-stamp: <00/03/06 13:24:13 teranisi>
+
+;; This file is part of ELMO (Elisp Library for Message Orchestration).
+
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+;;
+
+;;; Commentary:
+;; 
+
+;;; Code:
+;; 
+
+(eval-when-compile (require 'cl))
 (require 'elmo-vars)
 (require 'elmo-util)
 (require 'emu)
@@ -25,33 +38,38 @@
 (require 'elmo-cache)
 
 (defun elmo-msgdb-expand-path (folder &optional spec)
-  (elmo-convert-standard-filename
+  (convert-standard-filename
    (let* ((spec (or spec (elmo-folder-get-spec folder)))
-	  (type    (nth 0 spec))
-	  (fld-name (nth 1 spec))
-	  (server  (nth 2 spec))
-	  (user    (nth 3 spec)))
+	  (type (car spec))
+	  fld)
      (cond 
       ((eq type 'imap4)
+       (setq fld (elmo-imap4-spec-mailbox spec))
+       (if (string= "inbox" (downcase fld))
+	   (setq fld "inbox"))
+       (if (eq (string-to-char fld) ?/)
+	   (setq fld (substring fld 1 (length fld))))
        (expand-file-name 
-	fld-name
-	(expand-file-name (or user "nobody")
-			  (expand-file-name (or server "nowhere")
+	fld
+	(expand-file-name (or (elmo-imap4-spec-username spec) "nobody")
+			  (expand-file-name (or 
+					     (elmo-imap4-spec-hostname spec)
+					     "nowhere")
 					    (expand-file-name 
 					     "imap"
 					     elmo-msgdb-dir)))))
       ((eq type 'nntp)
        (expand-file-name 
-	fld-name
-	(expand-file-name (or server "nowhere")
+	(elmo-nntp-spec-group spec)
+	(expand-file-name (or (elmo-nntp-spec-hostname spec) "nowhere")
 			  (expand-file-name "nntp"
 					    elmo-msgdb-dir))))
-      ((eq type 'maildir) ; qmail Maildir... ???
-       (expand-file-name (elmo-safe-filename fld-name)
+      ((eq type 'maildir)
+       (expand-file-name (elmo-safe-filename (nth 1 spec))
 			 (expand-file-name "maildir"
 					   elmo-msgdb-dir)))
       ((eq type 'folder)
-       (expand-file-name (elmo-safe-filename fld-name)
+       (expand-file-name (elmo-safe-filename (nth 1 spec))
 			 (expand-file-name "folder"
 					   elmo-msgdb-dir)))
       ((eq type 'multi)
@@ -70,23 +88,24 @@
 	  (elmo-replace-in-string 
 	   (elmo-replace-in-string
 	    (elmo-replace-in-string 
-	     fld-name
+	     (nth 1 spec)
 	     "/" "_")
 	    ":" "__")
 	   "~" "___")
-	  "/" user))
-	(expand-file-name (concat (symbol-name type) "/" (symbol-name server))
+	  "/" (nth 3 spec)))
+	(expand-file-name (concat (symbol-name type) "/" 
+				  (symbol-name (nth 2 spec)))
 			  elmo-msgdb-dir)))
       ((eq type 'pop3)
        (expand-file-name 
-	(elmo-safe-filename fld-name)
-	(expand-file-name server
+	(elmo-safe-filename (elmo-pop3-spec-username spec))
+	(expand-file-name (elmo-pop3-spec-hostname spec)
 			  (expand-file-name
 			   "pop"
 			   elmo-msgdb-dir))))
       ((eq type 'localnews)
        (expand-file-name
-	(elmo-replace-in-string fld-name "/" ".")
+	(elmo-replace-in-string (nth 1 spec) "/" ".")
 	(expand-file-name "localnews"
 			  elmo-msgdb-dir)))
       ((eq type 'internal)
@@ -94,8 +113,16 @@
 						     (nth 2 spec)))
 			 (expand-file-name "internal"
 					   elmo-msgdb-dir)))
+      ((eq type 'cache)
+       (expand-file-name (elmo-safe-filename (nth 1 spec))
+			 (expand-file-name "internal/cache"
+					   elmo-msgdb-dir)))
       (t ; local dir or undefined type
-       (expand-file-name fld-name
+       ;; absolute path
+       (setq fld (nth 1 spec))
+       (if (file-name-absolute-p fld)
+	   (setq fld (elmo-safe-filename fld)))
+       (expand-file-name fld
 			 (expand-file-name (symbol-name type)
 					   elmo-msgdb-dir)))))))
 
@@ -125,8 +152,8 @@
     ret-val))
 
 ;;;
-;;; parsistent mark handling
-;;; (for global!)
+;; parsistent mark handling
+;; (for global!)
 
 (defvar elmo-msgdb-global-mark-alist nil)
 
@@ -154,8 +181,8 @@
     (if (setq match (assoc msgid malist))
 	(setcdr match mark)
       (setq elmo-msgdb-global-mark-alist
-	    (append elmo-msgdb-global-mark-alist
-		    (list (cons msgid mark)))))
+	    (nconc elmo-msgdb-global-mark-alist
+		   (list (cons msgid mark)))))
     (elmo-object-save path elmo-msgdb-global-mark-alist)))
 
 (defun elmo-msgdb-global-mark-get (msgid)
@@ -187,39 +214,42 @@
     elmo-msgdb-location-filename
     dir) alist))
 
-(defun elmo-list-folder-by-location (spec location &optional msgdb)
+(defun elmo-list-folder-by-location (spec locations &optional msgdb)
   (let* ((path (elmo-msgdb-expand-path nil spec))
-	 (location-alist 
-	  (if msgdb
-	      (elmo-msgdb-get-location msgdb)
-	    (elmo-msgdb-location-load path)))
-	 (i 0)
-	 result pair
-	 location-max modified)
-    (setq location-max
-	  (or (elmo-max-of-list (mapcar 'car location-alist)) 0))
-    (when location-max
-      (while location
-	(setq i (1+ i))
-	(if (setq pair (rassoc (car location) location-alist))
-	    (setq result
-		  (append result
-			  (list (cons (car pair) (car location)))))
-	  (setq result (append result
-			       (list 
-				(cons (+ location-max i) (car location))))))
-	(setq location (cdr location))))
-    (setq result (sort result (lambda (x y)
-				(< (car x)(car y)))))
-    (if (not (equal result location-alist))
-	(setq modified t))
-    (if modified
-	(elmo-msgdb-location-save path result))
+	 (location-alist (if msgdb
+			     (elmo-msgdb-get-location msgdb)
+			   (elmo-msgdb-location-load path)))
+	 (locations-in-db (mapcar 'cdr location-alist))
+	 result new-locs new-alist deleted-locs i
+	 modified)
+    (setq new-locs
+	  (elmo-delete-if (function
+			   (lambda (x) (member x locations-in-db)))
+			  locations))
+    (setq deleted-locs
+	  (elmo-delete-if (function
+			   (lambda (x) (member x locations)))
+			  locations-in-db))
+    (setq modified new-locs)
+    (setq i (or (elmo-max-of-list (mapcar 'car location-alist)) 0))
+    (mapcar 
+     (function 
+      (lambda (x)
+	(setq location-alist
+	      (delq (rassoc x location-alist) location-alist))))
+     deleted-locs)
+    (while new-locs
+      (setq i (1+ i))
+      (setq new-alist (cons (cons i (car new-locs)) new-alist))
+      (setq new-locs (cdr new-locs)))
+    (setq result (nconc location-alist new-alist))
+    (setq result (sort result (lambda (x y) (< (car x)(car y)))))
+    (if modified (elmo-msgdb-location-save path result))
     (mapcar 'car result)))
 
 ;;;
-;;; persistent mark handling
-;;; (for each folder)
+;; persistent mark handling
+;; (for each folder)
 (defun elmo-msgdb-mark-set (alist id mark)
   (let ((ret-val alist)
 	entity)
@@ -256,9 +286,6 @@
 ;;
 ;; overview handling
 ;;
-(if (fboundp 'std11-fetch-field)
-    (defalias 'elmo-field-body 'std11-fetch-field) ;;no narrow-to-region
-  (defalias 'elmo-field-body 'std11-field-body))
 
 (defsubst elmo-msgdb-get-field-value (field-name beg end buffer)
   (save-excursion
@@ -267,6 +294,45 @@
       (narrow-to-region beg end)
       (elmo-field-body field-name))))
 
+(defun elmo-multiple-field-body (name &optional boundary)
+  (save-excursion
+    (save-restriction
+      (std11-narrow-to-header boundary)
+      (goto-char (point-min))
+      (let ((case-fold-search t)
+	    (field-body nil))
+	(while (re-search-forward (concat "^" name ":[ \t]*") nil t)
+	  (setq field-body
+		(nconc field-body
+		       (list (buffer-substring-no-properties 
+			      (match-end 0) (std11-field-end))))))
+	field-body))))
+
+(defun elmo-multiple-fields-body-list (field-names &optional boundary)
+  "Return list of each field-bodies of FIELD-NAMES of the message header
+in current buffer. If BOUNDARY is not nil, it is used as message
+header separator."
+  (save-excursion
+    (save-restriction
+      (std11-narrow-to-header boundary)
+      (let* ((case-fold-search t)
+	     (s-rest field-names)
+	     field-name field-body)
+	(while (setq field-name (car s-rest))
+	  (goto-char (point-min))
+	  (while (re-search-forward (concat "^" field-name ":[ \t]*") nil t)
+	    (setq field-body
+		  (nconc field-body
+			 (list (buffer-substring-no-properties
+				(match-end 0) (std11-field-end))))))
+	  (setq s-rest (cdr s-rest)))
+	field-body))))
+
+(defsubst elmo-msgdb-remove-field-string (string)
+  (if (string-match (concat std11-field-head-regexp "[ \t]*") string)
+      (substring string (match-end 0))
+    string))
+
 (defsubst elmo-msgdb-get-last-message-id (string)
   (if string
       (save-match-data
@@ -277,7 +343,8 @@
 	   (when (search-backward "<" nil t)
 	     (setq beg (point))
 	     (if (search-forward ">" nil t)
-		 (buffer-substring beg (point)))))))))
+		 (elmo-replace-in-string
+		  (buffer-substring beg (point)) "\n[ \t]*" ""))))))))
 
 (defun elmo-msgdb-number-load (dir)
   (elmo-object-load 
@@ -311,30 +378,10 @@
    (expand-file-name elmo-msgdb-seen-filename dir)
    obj))
 
-(defun elmo-msgdb-overview-save (dir database)
-  (save-excursion
-    (let ((tmp-buffer (get-buffer-create " *Elmo Overview TMP*"))
-	  (ret-val t)
-	  write-file curp)
-      (set-buffer tmp-buffer)
-      (erase-buffer)
-      (setq curp database)
-      (prin1 database tmp-buffer)
-      (princ "\n" tmp-buffer)
-      (setq write-file (expand-file-name elmo-msgdb-overview-filename
-					 dir))
-      (if (file-directory-p dir)
-	  (); ok.
-	(if (file-exists-p dir)
-	    (error "File %s already exists." dir)
-	  (elmo-make-directory dir)))
-      (if (file-writable-p write-file)
-	  (as-binary-output-file (write-region (point-min) (point-max) 
-					       write-file nil 'no-msg))
-	(message (format "%s is not writable." write-file))
-	(setq ret-val nil))
-      (kill-buffer tmp-buffer)
-      ret-val)))
+(defsubst elmo-msgdb-overview-save (dir overview)
+  (elmo-object-save 
+   (expand-file-name elmo-msgdb-overview-filename dir)
+   overview))
 
 (defun elmo-msgdb-delete-msgs (folder msgs msgdb &optional reserve-cache)
   "Delete MSGS from FOLDER in MSGDB. 
@@ -383,21 +430,23 @@ content of MSGDB is changed."
 (defsubst elmo-msgdb-overview-entity-get-references (entity)
   (and entity (aref (cdr entity) 1)))
 
-;; id -> parent-entity
-(defsubst elmo-msgdb-overview-get-parent-entity (id database)
-  (let (reference)
-    (if (setq reference (elmo-msgdb-overview-entity-get-references 
-			 (assoc id database)))
-	(assoc reference database))))
+;; entity -> parent-entity
+(defsubst elmo-msgdb-overview-get-parent-entity (entity database)
+  (setq entity (elmo-msgdb-overview-entity-get-references entity))
+  ;; entity is parent-id.
+  (and entity (assoc entity database)))
   
 (defsubst elmo-msgdb-overview-entity-get-number (entity)
   (and entity (aref (cdr entity) 0)))
 
+(defsubst elmo-msgdb-overview-entity-get-from-no-decode (entity)
+  (and entity (aref (cdr entity) 2)))
+
 (defsubst elmo-msgdb-overview-entity-get-from (entity)
-  (and entity 
-       (let ((from (aref (cdr entity) 2)))
-	 (and from
-	      (decode-mime-charset-string from elmo-mime-charset)))))
+  (and entity
+       (aref (cdr entity) 2)
+       (decode-mime-charset-string (aref (cdr entity) 2) 
+				   elmo-mime-charset)))
 
 (defsubst elmo-msgdb-overview-entity-set-number (entity number)
   (and entity (aset (cdr entity) 0 number))
@@ -407,13 +456,15 @@ content of MSGDB is changed."
 (defsubst elmo-msgdb-overview-entity-set-from (entity from)
   (and entity (aset (cdr entity) 2 from))
   entity)
-  ;(setcar (car (cddadr entity)) from))
 
 (defsubst elmo-msgdb-overview-entity-get-subject (entity)
   (and entity
-       (let ((sub (aref (cdr entity) 3)))
-	 (and sub
-	      (decode-mime-charset-string sub elmo-mime-charset)))))
+       (aref (cdr entity) 3)
+       (decode-mime-charset-string (aref (cdr entity) 3)
+				   elmo-mime-charset)))
+
+(defsubst elmo-msgdb-overview-entity-get-subject-no-decode (entity)
+  (and entity (aref (cdr entity) 3)))
 
 (defsubst elmo-msgdb-overview-entity-set-subject (entity subject)
   (and entity (aset (cdr entity) 3 subject))
@@ -455,44 +506,14 @@ content of MSGDB is changed."
 ;; deleted message handling
 ;;
 (defun elmo-msgdb-killed-list-load (dir)
-  (save-excursion
-    (let ((tmp-buffer (get-buffer-create " *ELMO Killed TMP*"))
-	  (killed-list-file (expand-file-name elmo-msgdb-killed-filename dir))
-	  insert-file-contents-pre-hook   ; To avoid autoconv-xmas...
-	  insert-file-contents-post-hook 
-	  ret-val)
-      (if (not (file-readable-p killed-list-file))
-	  ()
-	(set-buffer tmp-buffer)
-	(insert-file-contents killed-list-file)
-	(setq ret-val
-	      (condition-case nil
-		  (read (current-buffer)) 
-		(error nil nil))))
-      (kill-buffer tmp-buffer)
-      ret-val)))
-;;
-;; currently not used...
-;;
+  (elmo-object-load
+   (expand-file-name elmo-msgdb-killed-filename dir)
+   nil t))
+
 (defun elmo-msgdb-killed-list-save (dir killed-list)
-  (save-excursion
-    (let ((tmp-buffer (get-buffer-create " *Elmo Killed TMP*"))
-	  (ret-val t)
-	  write-file)
-      (set-buffer tmp-buffer)
-      (erase-buffer)
-      (prin1 killed-list tmp-buffer)
-      (princ "\n" tmp-buffer)
-      (setq write-file (expand-file-name elmo-msgdb-killed-filename
-					 dir))
-      (if (file-writable-p write-file)
-	  (write-region (point-min) (point-max) 
-			write-file nil 'no-msg)
-	(message (format "%s is not writable." write-file))
-	(setq ret-val nil))
-      (kill-buffer tmp-buffer)
-      ret-val
-      )))
+  (elmo-object-save
+   (expand-file-name elmo-msgdb-killed-filename dir)
+   killed-list))
 
 (defun elmo-msgdb-killed-message-p (killed-list msg)
   (and killed-list
@@ -570,69 +591,43 @@ if integer, set number th message as killed."
 	  (setq ret-val (elmo-msgdb-append-element killed-list msg)))))
     ret-val))
 
-(defun elmo-msgdb-finfo-load (&optional filename)
-  (save-excursion
-    (let ((tmp-buffer (get-buffer-create " *ELMO Finfo TMP*"))
-	  (finfo-file (expand-file-name 
-		       (or filename elmo-msgdb-finfo-filename)
-		       elmo-msgdb-dir))
-	  insert-file-contents-pre-hook   ; To avoid autoconv-xmas...
-	  insert-file-contents-post-hook 
-	  ret-val)
-      (if (not (file-readable-p finfo-file))
-	  ()
-	(set-buffer tmp-buffer)
-	(insert-file-contents finfo-file)
-	(setq ret-val
-	      (condition-case nil
-		  (read (current-buffer)) 
-		(error nil nil))))
-      (kill-buffer tmp-buffer)
-      ret-val)))
+(defun elmo-msgdb-finfo-load ()
+  (elmo-object-load (expand-file-name 
+		     elmo-msgdb-finfo-filename
+		     elmo-msgdb-dir)
+		    elmo-mime-charset t))
 
-(defun elmo-msgdb-finfo-save (finfo &optional filename)
-  (save-excursion
-    (let ((tmp-buffer (get-buffer-create " *ELMO Finfo TMP*"))
-	  (finfo-file (expand-file-name
-		       (or filename elmo-msgdb-finfo-filename )
-		       elmo-msgdb-dir)))
-      (set-buffer tmp-buffer)
-      (erase-buffer)
-      (prin1 finfo tmp-buffer)
-      (princ "\n" tmp-buffer)
-      (if (file-directory-p elmo-msgdb-dir)
-	  (); ok.
-	(if (file-exists-p elmo-msgdb-dir)
-	    (error "File %s already exists." elmo-msgdb-dir)
-	  (elmo-make-directory elmo-msgdb-dir)))
-      (if (file-writable-p finfo-file)
-	  (write-region (point-min) (point-max) 
-			finfo-file nil 'no-msg)
-	(message (format "%s is not writable." finfo-file)))
-      (kill-buffer tmp-buffer))))
+(defun elmo-msgdb-finfo-save (finfo)
+  (elmo-object-save (expand-file-name
+		     elmo-msgdb-finfo-filename
+		     elmo-msgdb-dir)
+		    finfo elmo-mime-charset))
 
 (defun elmo-msgdb-flist-load (folder)
-  (save-excursion
-    (let ((tmp-buffer (get-buffer-create " *ELMO Flist TMP*"))
-	  (flist-file (expand-file-name 
-		       elmo-msgdb-flist-filename 
-		       (elmo-msgdb-expand-path folder (list 'folder folder))
-		       ))
-	  insert-file-contents-pre-hook   ; To avoid autoconv-xmas...
-	  insert-file-contents-post-hook
-	  ret-val)
-      (if (not (file-readable-p flist-file))
-	  ()
-	(set-buffer tmp-buffer)
-	(insert-file-contents flist-file)
-	(setq ret-val
-	      (condition-case nil
-		  (read (current-buffer)) 
-		(error nil nil))))
-      (kill-buffer tmp-buffer)
-      ret-val)))
+  (let ((flist-file (expand-file-name
+		     elmo-msgdb-flist-filename
+		     (elmo-msgdb-expand-path folder (list 'folder folder)))))
+    (elmo-object-load flist-file nil t)))
 
-(defsubst elmo-msgdb-create-overview-from-buffer (number &optional size)
+(defun elmo-msgdb-flist-save (folder flist)
+  (let ((flist-file (expand-file-name
+		     elmo-msgdb-flist-filename
+		     (elmo-msgdb-expand-path folder (list 'folder folder)))))
+    (elmo-object-save flist-file flist)))
+
+(defun elmo-crosspost-alist-load ()
+  (elmo-object-load (expand-file-name
+		     elmo-crosspost-alist-filename
+		     elmo-msgdb-dir)
+		    nil t))
+
+(defun elmo-crosspost-alist-save (alist)
+  (elmo-object-save (expand-file-name
+		     elmo-crosspost-alist-filename
+		     elmo-msgdb-dir)
+		    alist))
+
+(defsubst elmo-msgdb-create-overview-from-buffer (number &optional size time)
   "Create overview entity from current buffer. 
 Header region is supposed to be narrowed."
   (save-excursion
@@ -646,26 +641,16 @@ Header region is supposed to be narrowed."
 		 (elmo-field-body "in-reply-to"))
 		(elmo-msgdb-get-last-message-id
 		 (elmo-field-body "references"))))
-      (setq from (encode-mime-charset-string
-		  (elmo-eword-decode-string
-		   (decode-mime-charset-string
-		    (elmo-delete-char
-		     ?\"
-		     (or 
-		      (elmo-field-body "from")
-		      elmo-no-from))
-		    elmo-mime-charset))
-		  elmo-mime-charset))
-      (setq subject (encode-mime-charset-string
-		     (elmo-eword-decode-string
-		      (decode-mime-charset-string
-		       (or (elmo-field-body "subject")
-			   elmo-no-subject)
-		       elmo-mime-charset))
-		     elmo-mime-charset))
-      (setq date (elmo-field-body "date"))
-      (setq to   (elmo-field-body "to"))
-      (setq cc   (elmo-field-body "cc"))
+      (setq from (elmo-mime-string (elmo-delete-char
+				    ?\"
+				    (or 
+				     (elmo-field-body "from")
+				     elmo-no-from))))
+      (setq subject (elmo-mime-string (or (elmo-field-body "subject")
+					  elmo-no-subject)))
+      (setq date (or (elmo-field-body "date") time))
+      (setq to   (mapconcat 'identity (elmo-multiple-field-body "to") ","))
+      (setq cc   (mapconcat 'identity (elmo-multiple-field-body "cc") ","))
       (or size
 	  (if (setq size (elmo-field-body "content-length"))
 	      (setq size (string-to-int size))
@@ -680,28 +665,6 @@ Header region is supposed to be narrowed."
 			       size extra))
       )))
   
-(defun elmo-msgdb-flist-save (folder flist)
-  (save-excursion
-    (let* ((tmp-buffer (get-buffer-create " *ELMO Flist TMP*"))
-	   (dir (elmo-msgdb-expand-path folder
-					(list 'folder folder)))
-	   (flist-file (expand-file-name elmo-msgdb-flist-filename 
-					 dir)))
-      (set-buffer tmp-buffer)
-      (erase-buffer)
-      (prin1 flist tmp-buffer)
-      (princ "\n" tmp-buffer)
-      (if (file-directory-p dir)
-	  (); ok.
-	(if (file-exists-p dir)
-	    (error "File %s already exists." dir)
-	  (elmo-make-directory dir)))
-      (if (file-writable-p flist-file)
-	  (write-region (point-min) (point-max) 
-			flist-file nil 'no-msg)
-	(message (format "%s is not writable." flist-file)))
-      (kill-buffer tmp-buffer))))
-
 (defun elmo-msgdb-overview-sort-by-date (overview)
   (sort overview
 	(function
@@ -715,10 +678,10 @@ Header region is supposed to be narrowed."
 	     (error))))))
 
 (defun elmo-msgdb-sort-by-date (msgdb)
-  (message "sorting...")
+  (message "Sorting...")
   (let ((overview (elmo-msgdb-get-overview msgdb)))
     (setq overview (elmo-msgdb-overview-sort-by-date overview))
-    (message "sorting...done.")
+    (message "Sorting...done.")
     (list overview (nth 1 msgdb)(nth 2 msgdb))))
 
 (defsubst elmo-msgdb-search-overview-entity (number number-alist overview)
@@ -728,71 +691,30 @@ Header region is supposed to be narrowed."
 	(assoc message-id overview)
       (elmo-msgdb-overview-get-entity-by-number overview number))))
 
-(defun elmo-msgdb-upgrade-folder (folder &optional dir)
-  "A subroutine for msgdb-upgrade."
-  (let* ((path (or dir (elmo-msgdb-expand-path folder)))
-	 (number-alist (elmo-msgdb-number-load path))
-	 (overview (elmo-msgdb-overview-load path))
-	 (mark-alist (elmo-msgdb-mark-load path)))
-    (when (and number-alist
-	       overview)
-      (setq number-alist
-	    (mapcar
-	     (lambda (x)
-	       (if (stringp (car x))
-		   (cons (string-to-int (car x))
-			 (cdr x))
-		 (cons (car x) (cdr x))))
-	     number-alist))
-      (setq overview
-	    (mapcar (lambda (x)
-		      (if (vectorp (cdr x)) ; already 0.8
-			  x
-			(let ((env (nth 1 x)))
-			  (if (stringp (car env))
-			      ;; not converted yet.
-			      (cons (car x)
-				    (vector
-				     (string-to-int (car env)) ;number 0
-				     (nth 1 env)         ;reference 1 
-				     (car (nth 2 env))   ;from 2
-				     (nth 1 (nth 2 env)) ;subject 3
-				     (nth 2 (nth 2 env)) ;date 4
-				     (nth 3 (nth 2 env)) ;to 5
-				     (nth 4 (nth 2 env)) ;cc 6
-				     (string-to-int (or 
-						     (nth 5 (nth 2 env)) 
-						     "0"));size 7
-				     nil))
-			    ;; already integered
-			    (cons (car x)
-				  (vector
-				   (car env) ;number 0
-				   (nth 1 env) ;reference 1
-				   (car (nth 2 env)) ; from 2
-				   (nth 1 (nth 2 env)) ;subject 3
-				   (nth 2 (nth 2 env)) ;date 4
-				   (nth 3 (nth 2 env)) ;to 5
-				   (nth 4 (nth 2 env)) ;cc 6
-				   (nth 5 (nth 2 env)) ;size 7
-				   nil)))))) 
-		    overview))
-      (setq mark-alist
-	    (mapcar
-	     (lambda (x)
-	       (if (stringp (car x))
-		   (list (string-to-int (car x)) (nth 1 x))
-		 (list (car x)(nth 1 x))))
-	     mark-alist))
-      (elmo-msgdb-overview-save path overview)
-      (elmo-msgdb-number-save path number-alist)
-      (elmo-msgdb-mark-save path mark-alist))))
-
-(defun elmo-msgdb-append (msgdb msgdb-append)
+(defsubst elmo-msgdb-append (msgdb msgdb-append)
   (list 
-   (append (car msgdb) (car msgdb-append))
-   (append (cadr msgdb) (cadr msgdb-append))
-   (append (caddr msgdb) (caddr msgdb-append))
-   (append (cadddr msgdb) (cadddr msgdb-append))))
+   (nconc (car msgdb) (car msgdb-append))
+   (nconc (cadr msgdb) (cadr msgdb-append))
+   (nconc (caddr msgdb) (caddr msgdb-append))
+   (nconc (cadddr msgdb) (cadddr msgdb-append))))
+
+(defun elmo-msgdb-delete-path (folder &optional spec)
+  (let ((path (elmo-msgdb-expand-path folder spec)))
+    (if (file-directory-p path)
+	(elmo-delete-directory path t))))
+
+(defun elmo-msgdb-rename-path (old-folder new-folder &optional old-spec new-spec)
+  (let* ((old (directory-file-name (elmo-msgdb-expand-path old-folder old-spec)))
+	 (new (directory-file-name (elmo-msgdb-expand-path new-folder new-spec)))
+	 (new-dir (directory-file-name (file-name-directory new))))
+    (if (not (file-directory-p old))
+	()
+      (if (file-exists-p new)
+	  (error "already exists directory: %s" new)
+	(if (not (file-exists-p new-dir))
+	    (elmo-make-directory new-dir))
+	(rename-file old new)))))
 
 (provide 'elmo-msgdb)
+
+;;; elmo-msgdb.el ends here
