@@ -1,25 +1,38 @@
-;;;
 ;;; elmo-multi.el -- Multiple Folder Interface for ELMO.
-;;; Copyright (C) 1998 Yuuichi Teranishi <teranisi@gohome.org>
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;
+
+;; Copyright 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+
+;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;; Keywords: mail, net news
+;; Time-stamp: <00/03/14 19:41:07 teranisi>
+
+;; This file is part of ELMO (Elisp Library for Message Orchestration).
+
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+;;
+
+;;; Commentary:
+;; 
+
+;;; Code:
+;; 
 
 (require 'elmo-msgdb)
 (require 'elmo-vars)
+(require 'elmo2)
 
 (defun elmo-multi-msgdb (msgdb base)
   (list (mapcar (function
@@ -39,29 +52,6 @@
 			      (+ base (car x))
 			      (cdr x)))) (nth 2 msgdb))))
 
-(defun elmo-multi-get-number-alist-list (number-alist)
-  (let ((alist (sort number-alist (function (lambda (x y) (< (car x)
-							     (car y))))))
-	(cur-number 0)
-	one-alist ret-val num)
-    (while alist
-      (setq cur-number (+ cur-number 1))
-      (setq one-alist nil)
-      (while (and alist
-		  (eq 0
-		      (/ (- (setq num (car (car alist)))
-			    (* elmo-multi-divide-number cur-number))
-			 elmo-multi-divide-number)))
-	(setq one-alist (nconc
-			 one-alist 
-			 (list
-			  (cons
-			   (% num (* elmo-multi-divide-number cur-number))
-			   (cdr (car alist))))))
-	(setq alist (cdr alist)))
-      (setq ret-val (nconc ret-val (list one-alist))))
-    ret-val))
-
 (defun elmo-multi-msgdb-create-as-numlist (spec numlist new-mark already-mark
 						seen-mark important-mark
 						seen-list)
@@ -106,14 +96,14 @@
 	    ;; base is also same...delete it!
 	    (setq to-be-deleted (append to-be-deleted (list (car cur))))))
       (setq cur (cdr cur)))
-    (setq overview (delete-if 
+    (setq overview (elmo-delete-if 
 		    (function
 		     (lambda (x)
 		       (assq
 			(elmo-msgdb-overview-entity-get-number x)
 			to-be-deleted)))
 		    (elmo-msgdb-get-overview append-msgdb)))
-    (setq mark-alist (delete-if 
+    (setq mark-alist (elmo-delete-if 
 		      (function
 		       (lambda (x)
 			 (assq
@@ -150,12 +140,10 @@
 
 (defun elmo-multi-list-folders (spec &optional hierarchy)
   ;; not implemented.
-  nil
-  )
+  nil)
 
 (defun elmo-multi-append-msg (spec string)
-  (error "Cannot append messages to multi folder.")
-  )
+  (error "Cannot append messages to multi folder"))
 
 (defun elmo-multi-read-msg (spec number outbuf)
   (let* ((flds (cdr spec))
@@ -174,6 +162,66 @@
       (setq cur-number (+ 1 cur-number)))
     t))
 
+(defun elmo-multi-mark-alist-list (mark-alist)
+  (let ((cur-number 0)
+	one-alist result)
+    (while mark-alist
+      (setq cur-number (+ cur-number 1))
+      (setq one-alist nil)
+      (while (and mark-alist
+		  (eq 0
+		      (/ (- (car (car mark-alist))
+			    (* elmo-multi-divide-number cur-number))
+			 elmo-multi-divide-number)))
+	(setq one-alist (nconc
+			 one-alist 
+			 (list 
+			  (list (% (car (car mark-alist))
+				   (* elmo-multi-divide-number cur-number))
+				(cadr (car mark-alist))))))
+	(setq mark-alist (cdr mark-alist)))
+      (setq result (nconc result (list one-alist))))
+    result))
+
+(defun elmo-multi-list-folder-unread (spec mark-alist unread-marks)
+  (let* ((flds (cdr spec))
+	 (cur-number 0)
+	 mark-alist-list
+	 ret-val)
+    (setq mark-alist-list (elmo-multi-mark-alist-list mark-alist))
+    (while flds
+      (setq cur-number (+ cur-number 1))
+      (setq ret-val (append 
+		     ret-val
+		     (mapcar 
+		      (function
+		       (lambda (x)
+			 (+ 
+			  (* elmo-multi-divide-number cur-number) x)))
+		      (elmo-list-folder-unread (car flds)
+					       (car mark-alist-list) 
+					       unread-marks))))
+      (setq mark-alist-list (cdr mark-alist-list))
+      (setq flds (cdr flds)))
+    ret-val))
+
+(defun elmo-multi-list-folder-important (spec overview)
+  (let* ((flds (cdr spec))
+	 (cur-number 0)
+	 ret-val)
+    (while flds
+      (setq cur-number (+ cur-number 1))
+      (setq ret-val (append 
+		     ret-val
+		     (mapcar 
+		      (function
+		       (lambda (x)
+			 (+ 
+			  (* elmo-multi-divide-number cur-number) x)))
+		      (elmo-list-folder-important (car flds) overview))))
+      (setq flds (cdr flds)))
+    ret-val))
+
 (defun elmo-multi-list-folder (spec)
   (let* ((flds (cdr spec))
 	 (cur-number 0)
@@ -191,36 +239,38 @@
       (setq flds (cdr flds)))
     ret-val))
 
-(defun elmo-multi-folder-diff (fld)
-  (let ((flds (cdr (elmo-folder-get-spec fld)))
-	(num-alist-list
-	 (elmo-multi-get-number-alist-list 
-	  (elmo-msgdb-number-load (elmo-msgdb-expand-path fld))))
-	(count 0)
-	diffs (unsync 0) (nomif 0))
-    (while flds
-      (setq diffs (nconc diffs (list (elmo-folder-diff (car flds) 
-						       (nth count 
-							    num-alist-list)
-						       ))))
-      (setq count (+ 1 count))
-      (setq flds (cdr flds)))
-    (while diffs
-      (setq unsync (+ unsync (car (car diffs))))
-      (setq nomif  (+ nomif (cdr (car diffs))))
-      (setq diffs (cdr diffs)))
-    (elmo-folder-set-info-hashtb fld nil nomif)
-    (cons unsync nomif)))
-
 (defun elmo-multi-folder-exists-p (spec)
-  t 
-  )
+  (let* ((flds (cdr spec)))
+    (catch 'exists
+      (while flds
+	(unless (elmo-folder-exists-p (car flds))
+	  (throw 'exists nil))
+	(setq flds (cdr flds)))
+      t)))
+
+(defun elmo-multi-folder-creatable-p (spec)
+  (let* ((flds (cdr spec)))
+    (catch 'creatable
+      (while flds
+	(when (and (elmo-call-func (car flds) "folder-creatable-p")
+		   (not (elmo-folder-exists-p (car flds))))
+	      ;; If folder already exists, don't to `creatable'.
+	      ;; Because this function is called, when folder doesn't exists.
+	  (throw 'creatable t))
+	(setq flds (cdr flds)))
+      nil)))
 
 (defun elmo-multi-create-folder (spec)
-  ;;noop
-  )
+  (let* ((flds (cdr spec)))
+    (catch 'create
+      (while flds
+	(unless (or (elmo-folder-exists-p (car flds))
+		    (elmo-create-folder (car flds)))
+	  (throw 'create nil))
+	(setq flds (cdr flds)))
+      t)))
 
-(defun elmo-multi-search (spec key &optional numlist)
+(defun elmo-multi-search (spec condition &optional numlist)
   (let* ((flds (cdr spec))
 	 (cur-number 0)
 	 numlist-list cur-numlist ; for filtered search.
@@ -245,7 +295,7 @@
 			  (+ 
 			   (* elmo-multi-divide-number cur-number) x)))
 		       (elmo-call-func
-			(car flds) "search" key)))))
+			(car flds) "search" condition)))))
       (when numlist
 	(setq numlist-list (cdr numlist-list)))
       (setq flds (cdr flds)))
@@ -263,11 +313,53 @@
 		  "local-file-p" 
 		  (% number elmo-multi-divide-number)))
 
-(defun elmo-multi-get-msg-filename (spec number)
+(defun elmo-multi-commit (spec)
+  (mapcar 'elmo-commit (cdr spec)))
+
+(defun elmo-multi-plugged-p (spec)
+  (let* ((flds (cdr spec)))
+    (catch 'plugged
+      (while flds
+	(unless (elmo-folder-plugged-p (car flds))
+	  (throw 'plugged nil))
+	(setq flds (cdr flds)))
+      t)))
+
+(defun elmo-multi-set-plugged (spec plugged add)
+  (let* ((flds (cdr spec)))
+    (while flds
+      (elmo-folder-set-plugged (car flds) plugged add)
+      (setq flds (cdr flds)))))
+
+(defun elmo-multi-get-msg-filename (spec number &optional loc-alist)
   (elmo-call-func (nth (- (/ number elmo-multi-divide-number) 1)
 		       (cdr spec))
 		  "get-msg-filename"
-		  (% number elmo-multi-divide-number)))
+		  (% number elmo-multi-divide-number)
+		  loc-alist))
+
+(defun elmo-multi-sync-number-alist (spec number-alist)
+  (let ((folder-list (cdr spec))
+	(number-alist-list
+	 (elmo-multi-get-number-alist-list number-alist))
+	(multi-base 0)
+	append-alist result-alist)
+    (while folder-list
+      (incf multi-base)
+      (setq append-alist
+	    (elmo-call-func (nth (- multi-base 1) (cdr spec)) ;; folder name
+			    "sync-number-alist" 
+			    (nth (- multi-base 1) number-alist-list)))
+      (mapcar 
+       (function
+	(lambda (x)
+	  (setcar x 
+		  (+ (* elmo-multi-divide-number multi-base) (car x)))))
+       append-alist)
+      (setq result-alist (nconc result-alist append-alist))
+      (setq folder-list (cdr folder-list)))
+    result-alist))
 
 (provide 'elmo-multi)
 
+;;; elmo-multi.el ends here
