@@ -4,7 +4,7 @@
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
 ;; Keywords: mail, net news
-;; Time-stamp: <00/03/14 19:41:50 teranisi>
+;; Time-stamp: <00/04/28 10:30:51 teranisi>
 
 ;; This file is part of ELMO (Elisp Library for Message Orchestration).
 
@@ -411,7 +411,9 @@ Don't cache if nil.")
 		      (setq ret-val (nconc ret-val (list (list top-ng))))))
 		(setq i (1+ i))
 		(and (zerop (% i 10))
-		     (message "Parsing active... %d%%" (/ (* i 100) len)))
+		     (elmo-display-progress
+		      'elmo-nntp-list-folders "Parsing active..."
+		      (/ (* i 100) len)))
 		(forward-line 1)
 		))
 	  (while (re-search-forward "\\([^ ]+\\) .*\n" nil t)
@@ -419,8 +421,13 @@ Don't cache if nil.")
 				 (list (elmo-match-buffer 1))))
 	    (setq i (1+ i))
 	    (and (zerop (% i 10))
-		 (message "Parsing active... %d%%" (/ (* i 100) len))))))
+		 (elmo-display-progress
+		  'elmo-nntp-list-folders "Parsing active..."
+		  (/ (* i 100) len))))))
       (kill-buffer tmp-buffer)
+      (elmo-display-progress
+       'elmo-nntp-list-folders "Parsing active..."
+       100)
       (unless (string= server elmo-default-nntp-server)
 	(setq append-serv (concat "@" server)))
       (unless (eq port elmo-default-nntp-port)
@@ -676,10 +683,11 @@ Don't cache if nil.")
 	      (if (null (setq ov-str (elmo-nntp-read-contents buffer process)))
 		  (error "Fetching overview failed")))
 	    (setq cur (+ elmo-nntp-overview-fetch-chop-length cur 1))
-	    (message "Getting overview...%d%%" 
-		     (/ (* (+ (- (min cur
-				      end-num)
-				 beg-num) 1) 100) length))))
+	    (elmo-display-progress
+	     'elmo-nntp-msgdb-create "Getting overview..." 
+	     (/ (* (+ (- (min cur
+			      end-num)
+			 beg-num) 1) 100) length))))
 	(if (not use-xover)
 	    (setq ret-val (elmo-nntp-msgdb-create-by-header
 			   folder buffer process numlist
@@ -697,8 +705,9 @@ Don't cache if nil.")
 			seen-mark
 			important-mark
 			seen-list
-			filter)))))
-	  (message "Getting overview...done."))
+			filter))))))
+	(elmo-display-progress
+	 'elmo-nntp-msgdb-create "Getting overview..." 100)
 	;; If there are canceled messages, overviews are not obtained
 	;; to max-number(inn 2.3?).
 	(when (and (elmo-nntp-max-number-precedes-list-active-p)
@@ -1157,9 +1166,13 @@ Return nil if connection failed."
 	(accept-process-output process 1)
 	(discard-input)
 	(and (zerop (% received 10))
-	     (message "Getting folders info... %d%%"
-		      (/ (* received 100) count)))
+	     (elmo-display-progress
+	      'elmo-nntp-groups-read-response "Getting folders info..."
+	      (/ (* received 100) count)))
 	)
+      (elmo-display-progress
+       'elmo-nntp-groups-read-response "Getting folders info..."
+       100)
       ;; Wait for the reply from the final command.
       (goto-char (point-max))
       (re-search-backward "^[0-9]" nil t)
@@ -1235,11 +1248,14 @@ Return nil if connection failed."
 		     (setq received (1+ received)))
 		   (< received count))
 	    (and (zerop (% received 20))
-		 (message "Getting headers... %d%%"
-			  (/ (* received 100) number)))
+		 (elmo-display-progress
+		  'elmo-nntp-retrieve-headers "Getting headers..."
+		  (/ (* received 100) number)))
 	    (accept-process-output process 1)
 	    (discard-input)
 	    )))
+      (elmo-display-progress
+       'elmo-nntp-retrieve-headers "Getting headers..." 100)
       (message "Getting headers...done")
       ;; Remove all "\r"'s.
       (goto-char (point-min))
@@ -1300,10 +1316,11 @@ Return nil if connection failed."
 		))))
 	(setq i (1+ i))
 	(and (zerop (% i 20))
-	     (message "Creating msgdb...%d%%"
-		      (/ (* i 100) len)))
-	)
-      (message "Creating msgdb...done.")
+	     (elmo-display-progress
+	      'elmo-nntp-msgdb-create-message "Creating msgdb..."
+	      (/ (* i 100) len))))
+      (elmo-display-progress
+       'elmo-nntp-msgdb-create-message "Creating msgdb..." 100)
       (list overview number-alist mark-alist))))
 
 (defun elmo-nntp-use-cache-p (spec number)
