@@ -4,7 +4,7 @@
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
 ;; Keywords: mail, net news
-;; Time-stamp: <00/03/14 19:42:13 teranisi>
+;; Time-stamp: <00/04/28 10:28:08 teranisi>
 
 ;; This file is part of ELMO (Elisp Library for Message Orchestration).
 
@@ -208,7 +208,7 @@
 	(host server)
 	process-buffer ret-val response capability)
     (catch 'done
-;      (as-binary-process
+      (as-binary-process
        (setq process-buffer
 	     (get-buffer-create (format " *POP session to %s:%d" host port)))
        (save-excursion
@@ -341,7 +341,7 @@
 	 (when (null (elmo-pop3-read-response process-buffer process t))
 	   (setq ret-val (cons nil process))
 	   (throw 'done nil))
-	 (setq ret-val (cons process-buffer process))));)
+	 (setq ret-val (cons process-buffer process)))))
     ret-val))
 
 (defun elmo-pop3-read-contents (buffer process)
@@ -477,13 +477,15 @@
 		     (setq received (1+ received)))
 		   (< received count))
 	    (and (zerop (% received 20))
-		 (message "Getting headers... %d%%"
-			  (/ (* received 100) number)))
+		 (elmo-display-progress
+		  'elmo-pop3-retrieve-headers "Getting headers..."
+		  (/ (* received 100) number)))
 	    (accept-process-output process 1)
 	    ;(accept-process-output process)
 	    (discard-input)
 	    )))
-      (message "Getting headers...done")
+      (elmo-display-progress
+       'elmo-pop3-retrieve-headers "Getting headers..." 100)
       ;; Remove all "\r"'s.
       (goto-char (point-min))
       (while (search-forward "\r\n" nil t)
@@ -574,10 +576,12 @@
 	      )))
 	(setq i (1+ i))
 	(and (zerop (% i 20))
-	     (message "Creating msgdb...%d%%"
-		      (/ (* i 100) num)))
+	     (elmo-display-progress
+	      'elmo-pop3-msgdb-create-message "Creating msgdb..."
+	      (/ (* i 100) num)))
 	)
-      (message "Creating msgdb...done.")
+      (elmo-display-progress
+       'elmo-pop3-msgdb-create-message "Creating msgdb..." 100)
       (list overview number-alist mark-alist))))
 
 (defun elmo-pop3-read-body (buffer process outbuf)
