@@ -4,7 +4,7 @@
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
 ;; Keywords: mail, net news
-;; Time-stamp: <00/03/14 19:42:43 teranisi>
+;; Time-stamp: <2000-03-29 09:42:41 teranisi>
 
 ;; This file is part of ELMO (Elisp Library for Message Orchestration).
 
@@ -1148,7 +1148,8 @@ Otherwise treat \\ in NEWTEXT string as special:
 	      (setq i (+ i 1))
 	      (setq percent (/ (* i 100) num))
 	      (if (eq (% percent 5) 0)
-		  (message "%s%d%%" mes percent))))
+		  (elmo-display-progress
+		   'elmo-list-bigger-diff "%s%d%%" percent mes))))
 	(setq l1 (cdr l1)))
       (cons diff1 (list l2)))))
 
@@ -1477,6 +1478,15 @@ Otherwise treat \\ in NEWTEXT string as special:
 	(setq first nil))
       (princ err-mes stream))))
 
+(if (fboundp 'lprogress-display)
+    (defalias 'elmo-display-progress 'lprogress-display)
+  (defun elmo-display-progress (label format &optional value &rest args)
+    "Print a progress message."
+    (if (and (null format) (null args))
+	(message nil)
+      (apply (function message) (concat format " %d%%")
+	     (nconc args (list value))))))
+
 (defun elmo-time-expire (before-time diff-time)
   (let* ((current (current-time))
 	 (rest (when (< (nth 1 current) (nth 1 before-time))
